@@ -85,7 +85,7 @@
 		$err
 	);
 
-	our $VERSION = "2023.199.1";
+	our $VERSION = "2023.221.1";
 
 	our @EXPORT_OK = @EXPORT;
 
@@ -105,8 +105,8 @@
 	use constant SQL_SIMPLE_CURSOR_RELOAD => 5;	# page current
 
 	use constant SQL_SIMPLE_ORDER_OFF => undef;	# order disabled
-	use constant SQL_SIMPLE_ORDER_ASC => 1;		# order asceding
-	use constant SQL_SIMPLE_ORDER_DESC => 2;	# order descending
+	use constant SQL_SIMPLE_ORDER_ASC => "ASC";	# order asceding
+	use constant SQL_SIMPLE_ORDER_DESC => "DESC";	# order descending
 
 	use constant SQL_SIMPLE_CMD_OFF => 0;		# sql_save disabled
 	use constant SQL_SIMPLE_CMD_ON => 1;		# sql_save update
@@ -491,8 +491,9 @@ sub _SelectCursor()
 	## have order by?
 	my @order;
 	if	(!defined($argv->{order_by})) {}
-	elsif	(ref($argv->{order_by}) eq "") { @order = ($argv->{order_by}); }
 	elsif	(ref($argv->{order_by}) eq "ARRAY") { @order = @{$argv->{order_by}}; }
+	elsif	(ref($argv->{order_by}) eq "HASH") { @order = ($argv->{order_by}); }
+	elsif	(ref($argv->{order_by}) eq "") { @order = ($argv->{order_by}); }
 	else
 	{
 		$self->setMessage("select",SQL_SIMPLE_RC_SYNTAX,"009");
@@ -501,23 +502,23 @@ sub _SelectCursor()
 	## test the type of cursor
 	if	($argv->{cursor_command} == SQL_SIMPLE_CURSOR_TOP)
 	{
-		unshift(@order,$argv->{cursor_key} => SQL_SIMPLE_ORDER_ASC);
+		unshift(@order,{$argv->{cursor_key} => SQL_SIMPLE_ORDER_ASC});
 	}
 	elsif	($argv->{cursor_command} == SQL_SIMPLE_CURSOR_BACK)
 	{
-		unshift(@order,$argv->{cursor_key} => SQL_SIMPLE_ORDER_DESC);
+		unshift(@order,{$argv->{cursor_key} => SQL_SIMPLE_ORDER_DESC});
 
 		unshift(@{$argv->{where}}, $argv->{cursor_key} => [ "<", $argv->{cursor} ]);
 	}
 	elsif	($argv->{cursor_command} == SQL_SIMPLE_CURSOR_NEXT || $argv->{cursor_command} == SQL_SIMPLE_CURSOR_RELOAD)
 	{
-		unshift(@order,$argv->{cursor_key} => SQL_SIMPLE_ORDER_ASC);
+		unshift(@order,{$argv->{cursor_key} => SQL_SIMPLE_ORDER_ASC});
 
 		unshift(@{$argv->{where}}, $argv->{cursor_key} => [ ">", $argv->{cursor} ]);
 	}
 	elsif	($argv->{cursor_command} == SQL_SIMPLE_CURSOR_LAST)
 	{
-		unshift(@order,$argv->{cursor_key} => SQL_SIMPLE_ORDER_DESC);
+		unshift(@order,{$argv->{cursor_key} => SQL_SIMPLE_ORDER_DESC});
 	}
 	else
 	{
@@ -729,14 +730,9 @@ sub _Select()
 	if (defined($argv->{order_by}))
 	{
 		if	(ref($argv->{order_by}) eq "") { @order_work = ($argv->{order_by}); }
+		elsif	(ref($argv->{order_by}) eq "HASH") { @order_work = ($argv->{order_by}); }
 		elsif	(ref($argv->{order_by}) eq "ARRAY")
 		{
-			if (@{$argv->{order_by}} == 1) { @order_work = @{$argv->{order_by}}; }
-			if (@{$argv->{order_by}} % 2 != 0)
-			{
-				$self->setMessage("select",SQL_SIMPLE_RC_SYNTAX,"009");
-				return SQL_SIMPLE_RC_SYNTAX;
-			}
 		       	push(@order_work,@{$argv->{order_by}});
 		}
 		else
@@ -834,22 +830,14 @@ sub _Select()
 	while (@order_work)
 	{
 		my $field = shift(@order_work);
-		if ($field =~ /^(.*?)\.(.*?)$/)
+		if (ref($field) eq "HASH")
 		{
-			my $table = $1;
-			my $field = $2;
-			$field = $table.".".$self->getAliasCols($table,$field);
+			foreach my $_id(sort(keys(%{$field})))
+			{
+				$self->_SelectOrderBy(\@order,$table,$_id,$field->{$_id});
+			}
 		}
-		else
-		{
-			$field = $self->getAliasCols($table,$field);
-		}
-		if (@order_work)
-		{
-			my $ordem = shift(@order_work);
-			$field .= " ".(($ordem == SQL_SIMPLE_ORDER_DESC) ? 'DESC' : ($ordem == SQL_SIMPLE_ORDER_ASC) ? 'ASC' : $ordem);
-		}
-		push(@order,$field);
+		else { $self->_SelectOrderBy(\@order,$table,$field); }
 	}
 	## make tables
 	my @tables;
@@ -882,6 +870,30 @@ sub _Select()
 	return SQL_SIMPLE_RC_EMPTY;
 }
 
+sub _SelectOrderBy()
+{
+	my $self = shift;
+	my $array = shift;
+	my $table = shift;
+	my $field = shift;
+	my $order = shift;
+
+	if ($field =~ /^(.*?)\.(.*?)$/)
+	{
+		my $table = $1;
+		my $field = $2;
+		$field = $table.".".$self->getAliasCols($table,$field);
+	}
+	else { $field = $self->getAliasCols($table,$field); }
+
+       	if ($order)
+	{
+		$order = uc($order);
+		$field .= " ".(($order eq SQL_SIMPLE_ORDER_ASC) ? 'ASC' : ($order eq SQL_SIMPLE_ORDER_DESC) ? 'DESC' : 'USING '.$order);
+	}
+	push(@{$array},$field);
+}
+
 ################################################################################
 ## action: delete
 ## return:
